"""
-----------------------------------------------------------------------------------------
freqfilters module
-----------------------------------------------------------------------------------------
Provides basic frequency domain filters. 
"""

import numpy as np


def ideal_lpf(size: tuple, distance: float) -> np.ndarray:
    """Ideal low-pass filter in the frequency domain.

    Args:
        size (tuple): Size of the filter.

        distance (float): Distance to the center frequency.

    Returns:
        np.ndarray: Filter image in the frequency domain.
    """

    x_grid, y_grid = np.meshgrid(np.arange(size[1]), np.arange(size[0]))

    d = np.sqrt((x_grid - (size[1] // 2)) ** 2 + (y_grid - (size[0] // 2)) ** 2)

    return (d < distance) * 1


def butterworth_lpf(size: tuple, d0: float, order: int = 2) -> np.ndarray:
    """Butterworth low-pass filter in the frequency domain.

    Args:
        size (tuple): Size of the filter.

        d0 (float): Distance threshold.

        order (int, optional): Filter order.
        Defaults to 2.

    Returns:
        np.ndarray: Filter image in the frequency domain.
    """
    x_grid, y_grid = np.meshgrid(np.arange(size[1]), np.arange(size[0]))

    d = np.sqrt((x_grid - size[1] // 2) ** 2 + (y_grid - size[0] // 2) ** 2)

    return 1 / (1 + np.float_power(d / d0, 2 * order))


def gaussian_lpf(size: tuple, d0: float) -> np.ndarray:
    """Gaussian low-pass filter in the frequency domain.

    Args:
        size (tuple): Size of the filter.

        d0 (float): Distance threshold.

    Returns:
        np.ndarray: Filter image in the frequency domain.
    """
    x_grid, y_grid = np.meshgrid(np.arange(size[1]), np.arange(size[0]))

    d = np.sqrt((x_grid - size[1] // 2) ** 2 + (y_grid - size[0] // 2) ** 2)

    return np.exp((-(d**2)) / (2 * (d0**2)))



"""
-----------------------------------------------------------------------------------------
video module
-----------------------------------------------------------------------------------------
Provides basic video loading tools. 
"""

import os

import cv2 as cv
import numpy as np

def play_video(file_name: str) -> None:
    """Play videos with Open-CV

    Suppported formats: .avi
    Unsupported formats: .mov

    Args:
        fileName (str): Path to the file to be loaded.

    Raises:
        Exception: Errors related with file loading
    """
    _, ext = os.path.splitext(file_name)

    if ext != ".mov":  # Mov files give problems

        video = cv.VideoCapture(file_name)
        if not video.isOpened():
            raise Exception("Error loading file.")

        frameCount = int(video.get(cv.CAP_PROP_FRAME_COUNT))
        fps = int(video.get(cv.CAP_PROP_FPS))

        f = 0
        ret = True
        while f < frameCount and ret:
            ret, frame = video.read()
            if not ret:
                raise Exception("Error loading file.")

            cv.imshow("frame", frame)
            f += 1

            cv.waitKey(int(np.round(1e3 / fps)))

        video.release()
        cv.destroyAllWindows()
    else:
        raise Exception("%s format cannot be played" % ext)
    
def extract_video(file_name: str, resize: int = 1) -> tuple[np.ndarray, int]:
    """Extract videos with Open-CV and NumPy

    Suppported formats: .avi
    Unsupported formats: .mov

    Args:
        fileName (str): Path to the file to be loaded.

        resize (int, optional): Factor (>1) for resizing the video output.
        Defaults to 1 (no resizing).

    Raises:
        Exception: Errors related with file loading

    Returns:
        tuple[np.ndarray,int]: Results tuple:
            buf (element 0 np.ndarray): 4-dimensional array including the video.
            fps (element 1 int): Frames per second of the exported video.
    """
    _, ext = os.path.splitext(file_name)

    if ext != ".mov":  # Mov files give problems

        video = cv.VideoCapture(file_name)
        if not video.isOpened():
            raise Exception("Error loading file.")

        frameCount = int(video.get(cv.CAP_PROP_FRAME_COUNT))
        frameWidth = int(video.get(cv.CAP_PROP_FRAME_WIDTH)) // resize
        frameHeight = int(video.get(cv.CAP_PROP_FRAME_HEIGHT)) // resize
        fps = int(video.get(cv.CAP_PROP_FPS))

        buf = np.empty((frameCount, frameHeight, frameWidth, 3), np.dtype("uint8"))

        ret = True
        frame = 0
        while frame < frameCount and ret:
            ret, image = video.read()
            if not ret:
                raise Exception("Error loading file.")

            buf[frame] = cv.resize(
                cv.cvtColor(image, cv.COLOR_BGR2RGB), (frameWidth, frameHeight)
            )
            frame += 1

        video.release()
    else:
        raise Exception("%s format cannot be read." % ext)

    return (buf, fps)


from pathlib import Path
from setuptools import setup, find_packages
import re
import os


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

DESCRIPTION = "A Python package to provide easy to use tools to learn and teach digital (biomedical) image processing with Python."
PACKAGE_NAME = "diptools"
AUTHOR = "Alejandro Alcaine, PhD"
EMAIL = "lalcaine@usj.es"
GITHUB_URL = "https://github.com/aalcaineo/diptools"

with open(os.path.join(this_directory,PACKAGE_NAME,"__init__.py"), "r") as f:
    version = ""
    while not version:
        version = re.findall('\t*\s*^__version__\s*=\s*"(\d*\.\d*\.\d*)"\n+', f.readline())

setup(
    name="usj_" + PACKAGE_NAME.lower(), # To be compliant with PEP 625
    packages=find_packages(exclude=['*tests*']),
    version=version[0],
    license="GNU General Public License v2.0",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    author=AUTHOR,
    author_email=EMAIL,
    url=GITHUB_URL,
    keywords=["Digital Signal Processing"],
    install_requires=["matplotlib>=3.7.0","numpy>=1.23.5","opencv-python>=4.9.0"],
    python_requires=">=3.10",
    extras_require={"dev":["twine>=4.0.2"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3",
    ],
    zip_safe=False,
)



"""
-----------------------------------------------------------------------------------------
DIPTools
-----------------------------------------------------------------------------------------
Digital Image Processing Tools is a Python package to provide easy to use tools to learn
and teach digital (biomedical) image processing with Python.

Version: 2025.03.14

### Modules:
    * bitlevel: Provides functions for exploring bit-level representations of images. 
    * filters: Provides basic spatial filters. 
    * freqfilters: Provides basic frequency domain filters. 
    * graph: Provides functionality to easily visualise images. Functions `show_image()`
             and `show_histogram()` are already available in the namespace. 
    * pointprocessing: Provides point processing functionality. Function 
                       `plot_transformation()` is already available in the namespace.
    * region: Provides region based segmentation algorithms. Functions `region_filling()`
             and `region_growing()` are already available in the namespace. 
    * video: Provides basic video loading tools.

### Installation and usage:

Please install this package using PiP by typing:

`pip install USJ_diptools`

Import this module as:

`import diptools as dip`

### Author:
Alejandro Alcaine, Ph.D\\
CoMBA research group\\
MESC Working Group on e-Cardiology\\
MESC European Association of Cardiovascular Imaging (EACVI)\\
lalcaine@usj.es

Faculty of Health Sciences\\
University San Jorge\\
Villanueva de Gállego (Zaragoza)\\
Spain
"""

__version__ = "2025.03.14"

from diptools.graph import show_image, show_histogram 
from diptools.pointprocessing import plot_transformation
from diptools.region import region_filling, region_growing

import diptools.bitlevel
import diptools.filters
import diptools.freqfilters
import diptools.pointprocessing
import diptools.region
import diptools.video